<?php

App::uses('AppComponent', 'Controller/Component');

class ProductsComponent extends AppComponent {

	/**
	 * get a product listing
	 *
	 * @param 	array 	query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @param   array   An array of order data (direction and currencyId)
	 * @return 	array 	Array of results
	 */
	public function listing($params = array(), $paginate = true, $orderByPrice = null) {
		$Product = EvClassRegistry::init('EvShop.Product');
		$params = Hash::merge(
			array(
				'joins' => array(
					array(
						'table' => 'ev_shop_categories_products',
						'alias' => 'CategoriesProduct',
						'conditions' => array(
							'Product.id = CategoriesProduct.product_id'
						)
					)
				),
				'conditions' => array(
					'Product.is_active' => 1
				),
				'order' => 'CategoriesProduct.sequence ASC, Product.name ASC',
				'contain' => array(
					'ListingImage',
					'Variant' => array(
						'order' => 'Variant.sequence ASC',
						'VariantPricing'
					)
				),
				'group' => 'Product.id'
			),
			$params
		);

		if ($orderByPrice !== null && !empty($orderByPrice)) {
			$params = $this->_addPriceParamsToListing($params, $orderByPrice, $Product);
		}

		if ($paginate && ! empty($this->_controller)) {
			$this->_controller->Paginator->settings = $params;
			$result = $this->_controller->Paginator->paginate(
				$Product
			);
		} else {
			$result = $Product->find('all', $params);
		}

		$VariantModel = EvClassRegistry::init('EvShop.Variant');
		$VariantModel->linkInventory();

		$this->_addTaxToProducts($result);

		return $result;
	}

	/**
	 * get a featured product listing
	 *
	 * @param 	array 	query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @return 	array 	Array of results
	 */
	public function featured($params = array(), $paginate = false) {
		$params = Hash::merge(
			array(
				'conditions' => array(
					'Product.is_featured' => 1
				)
			),
			$params
		);

		return $this->listing($params, $paginate);
	}

	/**
	 * get a listing by category
	 *
	 * @param 	int|array 	Category ID or array of category IDs
	 * @param 	array 		query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @param   array       An array to pass through to listing that if set will contain the direction of ordering and currency_id
	 * @return 	array 		Array of results
	 */
	public function listingByCategory($categoryId, $params = array(), $paginate = true, $orderByPrice = null) {
		$defaults = array(
			'conditions' => array(
				'CategoriesProduct.category_id' => $categoryId
			)
		);
		// if it's many categories, and the group by and9
		// also contain the categories so we know which is which
		if (is_array($categoryId)) {
			$defaults['contain']['CategoriesProduct'] = array(
				'limit' => 1,
				'Category'
			);
		}

		$params = Hash::merge(
			$defaults,
			$params
		);

		return $this->listing($params, $paginate, $orderByPrice);
	}

	/**
	 * get a listing by a brand
	 *
	 * @param 	int|array 	brand ID or array of brand IDs
	 * @param 	array 		query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @return 	array 		Array of results
	 */
	public function listingByBrand($brandId, $params = array(), $paginate = true) {
		$defaults = array(
			'conditions' => array(
				'Product.brand_id' => $brandId
			)
		);

		// if it's many categories, and the group by and
		// also contain the categories so we know which is which
		if (is_array($brandId)) {
			$defaults['contain'][] = 'Brand';
		}

		$params = Hash::merge(
			$defaults,
			$params
		);

		return $this->listing($params, $paginate);
	}

	/**
	 * Adds joins, order and virtual fields to the pagination query parameters. Defaults to ordering
	 * in ascending order. The ordering will chose the lowest price between the price and sale_price
	 * to order by.
	 * @param array $currentParams The current parameters of the query to change. Normally the defaults set in listing()
	 * @param array $orderData     The ordering data (direction and currencyId if it is set)
	 * @param Model $queryModel    The model that is being queried on, listing uses Product
	 */
	protected function _addPriceParamsToListing($currentParams, $orderData, $queryModel) {
		list($priceFullField, $priceDirection) = explode(' ', $currentParams['order'], 2);
		list($priceModel, $priceField) = explode('.', $currentParams['order'], 2);

		$priceDirection = strtoupper($priceDirection);

		$currentParams['joins'][] = [
			'table' => 'ev_shop_variants',
			'alias' => 'Variant',
			'conditions' => [
				'Variant.product_id = Product.id'
			]
		];

		$currencyFilter = '';
		if (isset($orderData['currencyId'])) {
			$currencyFilter = ['VariantPricing.currency_id' => $orderData['currencyId']];
		}

		$currentParams['joins'][] = [
			'table' => 'ev_shop_variant_pricings',
			'alias' => 'VariantPricing',
			'conditions' => [
				'VariantPricing.variant_id = Variant.id',
				$currencyFilter
			],
			'type' => 'left'
		];

		$orderDirection = 'ASC';
		if (isset($orderData['direction'])) {
			$orderDirection = $orderData['direction'];
		}

		$currentParams['order'] = "MIN(IF(VariantPricing.sale_price IS NULL, VariantPricing.price, VariantPricing.sale_price)) $orderDirection";

		$queryModel->virtualFields['price'] = 'MIN(VariantPricing.price)';
		$queryModel->virtualFields['sale_price'] = 'MIN(VariantPricing.sale_price)';
		$queryModel->virtualFields['cheapest_price'] = 'MIN(IF(VariantPricing.sale_price IS NULL, VariantPricing.price, VariantPricing.sale_price))';

		return $currentParams;
	}

	/**
	 * Adds the _incTax fields to the query results. Checks that the EvTax plugin is loaded if not then
	 * it is assumed that the tax will be added manually from whatever peeps be using.
	 * @param array $products The results of the product query containing each product to add tax to.
	 */
	protected function _addTaxToProducts($products) {
		if (CakePlugin::loaded('EvTax')) {
			$TaxLevelModel = EvClassRegistry::init('EvTax.TaxLevel');

			$VariantModel = EvClassRegistry::init('EvShop.Variant');
			$VariantPricingModel = EvClassRegistry::init('EvShop.VariantPricing');
			$VariantModel->linkInventory();

			if (! empty($products)) {
				foreach ($products as $key => $item) {
					$taxLevel = array();
					if (isset($item['Product']['tax_level_id']) && $item['Product']['tax_level_id'] > 0) {

						$taxLevel = $TaxLevelModel->find('first', array(
							'conditions' => array(
								'id' => $item['Product']['tax_level_id']
							)
						));

						if (isset($item['Variant']) && ! empty($item['Variant'])) {
							foreach ($item['Variant'] as $vKey => $variant) {

								if (! empty($taxLevel['TaxLevel'])) {

									if (isset($variant['VariantPricing']) && ! empty($variant['VariantPricing'])) {

										foreach ($variant['VariantPricing'] as $pricingKey => $pricing) {
											$variant['VariantPricing'][$pricingKey]['TaxLevel'] = $taxLevel['TaxLevel'];
										}
									}
								}

								if (isset($result[$key]['Variant'][$vKey])) {
									$products[$key]['Variant'][$vKey] = $VariantPricingModel->calculateTax($variant, 'Variant');
								}
							}
						}
					}
				}
			}
		}

		return $products;
	}
}
